@@ -1,7 +1,7 @@
 ################################################################################
 ### Initialization and other basic methods for the S4 class "sts"
 ###
-### Copyright (C) 2007-2014 Michael Hoehle, 2012-2019 Sebastian Meyer
+### Copyright (C) 2007-2014 Michael Hoehle, 2012-2019,2021,2023 Sebastian Meyer
 ###
 ### This file is part of the R package "surveillance",
 ### free software under the terms of the GNU General Public License, version 2,
@@ -34,10 +34,11 @@ sts <- function (observed,
                  start = c(2000, 1), frequency = 52, # prototype values
                  epoch = NULL, # defaults to 1:nrow(observed), can be Date
                  population = NULL, # an alias for "populationFrac"
+                 map = NULL, # allow sf input for the map slot
                  ...) # further named arguments representing "sts" slots
 {
     slots <- list(observed = observed, start = start, freq = frequency,
-                  epoch = epoch, ...)
+                  epoch = epoch, map = map, ...)
 
     if (!is.null(population)) {
         if ("populationFrac" %in% names(slots))
@@ -53,6 +54,17 @@ sts <- function (observed,
         slots$epochAsDate <- TRUE
     }
 
+    if (inherits(map, "sf")) {
+        if (!requireNamespace("sf", quietly = TRUE)) {
+            stop("'map' is of class \"sf\" but package 'sf' is not installed.\n",
+                 "  Alternatively, use a map of class \"SpatialPolygons\" (from 'sp').")
+        }
+        ## allow "sf" input and transform it to "SpatialPolygons*"
+        slots$map <- sf::as_Spatial(map)
+    } else if (!is.null(map) && !inherits(map, "SpatialPolygons")) {
+        stop("'map' must inherit from \"sf\" or \"SpatialPolygons\"")
+    }
+
     ## call the standard generator function with explicitly set slots
     isNULL <- vapply(X = slots, FUN = is.null,
                      FUN.VALUE = FALSE, USE.NAMES = FALSE)
@@ -140,8 +152,9 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
                               neighbourhood=neighbourhood,
                               populationFrac=populationFrac)
     ## this also checks validObject(.Object)
+    ## for nUnit > 1, it will catch if any column names differ from namesObs
 
-    ## make sure all arrays have the same dimnames
+    ## use dimnames(observed) for all matrix slots (could be unnamed)
     .Object <- fix.dimnames(.Object)
 
     return(.Object)
@@ -171,9 +184,13 @@ disProg2sts <- function(disProgObj, map=NULL) {
 
 ## The reverse action
 sts2disProg <- function(sts) {
-  disProgObj <- create.disProg(week=sts@epoch, start=sts@start, freq=sts@freq,
-                               observed=sts@observed, state=sts@state, neighbourhood=sts@neighbourhood,
-                               populationFrac=sts@populationFrac, epochAsDate=sts@epochAsDate)
+  disProgObj <- list(  # no need for create.disProg() checks, sts is formal
+    week=sts@epoch, observed=sts@observed, state=sts@state,
+    start=sts@start, freq=sts@freq,
+    neighbourhood=sts@neighbourhood, populationFrac=sts@populationFrac,
+    epochAsDate=sts@epochAsDate
+  )
+  class(disProgObj) <- "disProg"
   #For survRes: alarm=sts@alarm, upperbound=sts@upperbound)
   return(disProgObj)
 }
@@ -266,12 +283,15 @@ setMethod("dimnames", "sts", function (x) dimnames(x@observed))
 
 #Extract which observation within year we have
 setMethod("epochInYear", "sts", function(x,...) {
-  if (x@epochAsDate) {
+  if (x@epochAsDate && x@freq %in% c(12, 52, 365)) {
     epochStr <- switch(as.character(x@freq),
                        "12" = "%m", "52" = "%V", "365" = "%j")
     as.numeric(strftime(epoch(x), epochStr))
   } else {
-    (x@epoch-1 + x@start[2]-1) %% x@freq + 1
+    index <- if (x@epochAsDate) { # non-standard frequency
+                 seq_along(x@epoch)
+             } else x@epoch  # should always be 1:nrow(x) actually
+    (index-1 + x@start[2]-1) %% x@freq + 1
   }
 })
 
@@ -440,17 +460,18 @@ setMethod( "show", "sts", function( object ){
   print(head(object@observed,n))
 
   if (npoly <- length(object@map)) {
-      cat("\nmap:\n")
-      print(modifyList(summary(object@map), list(data=NULL))) # no data summary
-      cat("Features    :", npoly, "\n")
-      if (inherits(object@map, "SpatialPolygonsDataFrame"))
-          cat("Data slot   :", ncol(object@map), "variables\n")
+      cat("\nmap:", npoly, "Polygons, ")
+      ## no longer print SpatialPolygons as this may load heavy sf (for is.projected)
+      ## print(modifyList(summary(object@map), list(data=NULL))) # no data summary
+      if (inherits(object@map, "SpatialPolygonsDataFrame")) {
+          cat(ncol(object@map), "variables")
+          cat("\nHead of map@data:\n")
+          print(head(object@map@data, n))
+      } else cat("without data\n")
   }
 
-  if (ncol(object@observed) > 1) {
-      cat("\nhead of neighbourhood:\n")
+  if (ncol(object@observed) > 1 && !all(is.na(object@neighbourhood))) {
+      cat("\nHead of neighbourhood:\n")
       print( head(object@neighbourhood,n))
   }
 } )
-
-
